/*
Copyright 2010 James Pether Sörling Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License. 
	$Id
*/

package com.hack23.cia.model.sweden.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinTable;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;
import javax.persistence.Version;

import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

import com.hack23.cia.model.core.impl.BaseEntity;
import com.hack23.cia.model.sweden.impl.Vote.Position;

/**
 * The Class Ballot.
 */
@Entity
@Table(uniqueConstraints = { @UniqueConstraint(columnNames = { "voteResultsHref" }) })
@Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
public class Ballot extends BaseEntity {

    /**
     * The Enum ImportStatus.
     */
    public enum ImportStatus {

        /** The Completed. */
        Completed,

        /** The Created. */
        Created
    }

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /** The Constant TOTAL_VOTES. */
    public static final int TOTAL_VOTES = 349;

    /** The ballot result. */
    private BallotResult ballotResult;

    /** The commitee report. */
    private CommitteeReport commiteeReport;

    /** The datum. */
    private Date datum;

    /** The description. */
    private String description;

    /** The id. */
    private Long id;

    /** The import status. */
    private ImportStatus importStatus = ImportStatus.Created;

    /** The overview href. */
    private String overviewHref;

    /** The party ballot result. */
    private List<PartyBallotResult> partyBallotResult = new ArrayList<PartyBallotResult>();

    /** The version. */
    private Long version=1L;

    /** The vote results href. */
    private String voteResultsHref;

    /** The votes. */
    private List<Vote> votes = new ArrayList<Vote>();
    
    /**
     * Gets the ballot result.
     *
     * @return the ballot result
     */
    @ManyToOne(cascade=CascadeType.ALL)
	public BallotResult getBallotResult() {
		return ballotResult;
	}

    /**
     * Gets the commitee report.
     *
     * @return the commitee report
     */
    @ManyToOne
    public CommitteeReport getCommiteeReport() {
        return commiteeReport;
    }

    /**
     * Gets the datum.
     *
     * @return the datum
     */
    @Temporal(TemporalType.DATE)
    public Date getDatum() {
        return datum;
    }

    /**
     * Gets the description.
     *
     * @return the description
     */
    public String getDescription() {
        return description;
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.hack23.cia.model.core.BaseEntity#getId()
     */
    @Override
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    public Long getId() {
        return id;
    }

    /**
     * Gets the import status.
     *
     * @return the import status
     */
    @Enumerated(EnumType.STRING)
    public ImportStatus getImportStatus() {
        return importStatus;
    }

    /**
     * Gets the overview href.
     *
     * @return the overview href
     */
    public String getOverviewHref() {
        return overviewHref;
    }

    /**
     * Gets the party ballot result.
     *
     * @return the party ballot result
     */
    @OneToMany(cascade = CascadeType.ALL, fetch = FetchType.LAZY)
    @JoinTable(name = "BallotPartyVoteResult")
	public List<PartyBallotResult> getPartyBallotResult() {
		return partyBallotResult;
	}

    /**
     * Gets the party ballot result.
     *
     * @param party the party
     * @return the party ballot result
     */
	@Transient
	public PartyBallotResult getPartyBallotResult(String party) {
		for(PartyBallotResult partyBallotResult : getPartyBallotResult()) {
			if (partyBallotResult.getPoliticalParty().getShortCode().equals(party)) {
				return partyBallotResult;
			}
		}
		return null;
	}

    /* (non-Javadoc)
     * @see com.hack23.cia.model.core.BaseEntity#getVersion()
     */
    @Override
    @Version
    public Long getVersion() {
        return version;
    }

    /**
     * Gets the vote for parliament member.
     *
     * @param parliamentMember the parliament member
     * @return the vote for parliament member
     */
    @Transient
    public Vote getVoteForParliamentMember(ParliamentMember parliamentMember) {
        Vote result = null;
        for (Vote vote : getVotes()) {
            if (vote.getParliamentMember().getId().equals(
                    parliamentMember.getId())) {
                return vote;
            }
        }
        return result;
    }

    /**
     * Gets the vote results href.
     *
     * @return the vote results href
     */
    public String getVoteResultsHref() {
        return voteResultsHref;
    }

    /**
     * Gets the votes.
     *
     * @return the votes
     */
    @OneToMany(mappedBy = "ballot", cascade = CascadeType.ALL, fetch = FetchType.LAZY)
    @Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE)
    public List<Vote> getVotes() {
        return votes;
    }

    /**
     * Checks if is rebel vote.
     *
     * @param vote the vote
     * @return true, if is rebel vote
     */
	@Transient
	public boolean isRebelVote(Vote vote) {
		return getPartyBallotResult(vote.getParliamentMember().getParty()).isRebelVote(vote);
	}

    /**
     * Checks if is winning position.
     *
     * @param position the position
     * @return true, if is winning position
     */
	@Transient
	public boolean isWinningPosition(Position position) {
		return getBallotResult().getWinningPosition().equals(position);
	}

    /**
     * Sets the ballot result.
     *
     * @param ballotResult the new ballot result
     */
	public void setBallotResult(BallotResult ballotResult) {
		this.ballotResult = ballotResult;
	}

    /**
     * Sets the commitee report.
     *
     * @param commiteeReport the new commitee report
     */
    public void setCommiteeReport(CommitteeReport commiteeReport) {
        this.commiteeReport = commiteeReport;
    }

    /**
     * Sets the datum.
     *
     * @param datum the new datum
     */
    public void setDatum(Date datum) {
        this.datum = datum;
    }
    
    /**
     * Sets the description.
     *
     * @param description the new description
     */
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * Sets the id.
     *
     * @param id the new id
     */
    public void setId(Long id) {
        this.id = id;
    }

	/**
	 * Sets the import status.
	 *
	 * @param importStatus the new import status
	 */
    public void setImportStatus(ImportStatus importStatus) {
        this.importStatus = importStatus;
    }

	/**
	 * Sets the overview href.
	 *
	 * @param overviewHref the new overview href
	 */
    public void setOverviewHref(String overviewHref) {
        this.overviewHref = overviewHref;
    }

	/**
	 * Sets the party ballot result.
	 *
	 * @param partyBallotResult the new party ballot result
	 */
	public void setPartyBallotResult(List<PartyBallotResult> partyBallotResult) {
		this.partyBallotResult = partyBallotResult;
	}

	/**
	 * Sets the version.
	 *
	 * @param version the new version
	 */
    public void setVersion(Long version) {
        this.version = version;
    }

    /**
     * Sets the vote results href.
     *
     * @param voteResultsHref the new vote results href
     */
    public void setVoteResultsHref(String voteResultsHref) {
        this.voteResultsHref = voteResultsHref;
    }

	/**
	 * Sets the votes.
	 *
	 * @param votes the new votes
	 */
    public void setVotes(List<Vote> votes) {
        this.votes = votes;
    }

}
