/*
Copyright 2010 James Pether Sörling Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License. 
	$Id
*/

package com.hack23.cia.service.impl.agent.sweden;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.html.HtmlAnchor;
import com.gargoylesoftware.htmlunit.html.HtmlElement;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlTable;
import com.gargoylesoftware.htmlunit.html.HtmlTableRow;
import com.hack23.cia.model.sweden.impl.CommitteeReport;

/**
 * The Class CommitteeReportAgentImpl.
 */
public class CommitteeReportAgentImpl implements CommitteeReportAgent {

    /**
     * The Class DocumentAnswerPage.
     */
    class DocumentAnswerPage {

        /** The anchors. */
        private List<HtmlAnchor> anchors;;

        /** The next page link row. */
        private HtmlTableRow nextPageLinkRow = null;

        /**
         * Instantiates a new document answer page.
         *
         * @param page the page
         */
        public DocumentAnswerPage(final HtmlPage page) {
            try {
                HtmlElement answerDiv = page.getHtmlElementById(ANSWER);
                Iterator<HtmlElement> iterator = answerDiv
                        .getHtmlElementsByTagName(TABLE).iterator();
                if (iterator.hasNext()) {
                    HtmlTable table = (HtmlTable) iterator.next();

                    anchors = table.getHtmlElementsByTagName(ANCHOR);

                    List<HtmlTableRow> rows = new ArrayList<HtmlTableRow>(table
                            .getRows());
                    rows.remove(0);
                    nextPageLinkRow = rows.remove(0);
                    rows.remove(rows.size() - 1);
                    return;
                } else {
                    LOGGER
                            .warn("Problem with page : " + page.getPage().getTitleText() + "\n\n" + page.asXml()); //$NON-NLS-1$
                }
            } catch (Exception e) {
                LOGGER.warn(e);
            }
        }

        /**
         * Gets the anchors.
         *
         * @return the anchors
         */
        public List<HtmlAnchor> getAnchors() {
            return anchors;
        }

        /**
         * Gets the next page.
         *
         * @return the next page
         * @throws Exception the exception
         */
        public DocumentAnswerPage getNextPage() throws Exception {
            if (nextPageLinkRow != null) {
                List<HtmlAnchor> anchors = nextPageLinkRow
                        .getHtmlElementsByTagName(ANCHOR);

                for (HtmlAnchor anchor : anchors) {
                    if (NEXT.equals(anchor.asText())
                            || NEXT_VERSION2.equals(anchor.asText())) {
                        return new DocumentAnswerPage((HtmlPage) anchor.click());
                    }
                }
            }
            return null;
        }
    }

    /** The Constant ANCHOR. */
    private static final String ANCHOR = "a"; //$NON-NLS-1$

    /** The Constant ANSWER. */
    private static final String ANSWER = "svar"; //$NON-NLS-1$

    /** The Constant CENTER_PADDING. */
    private static final String CENTER_PADDING = "centerPadding"; //$NON-NLS-1$

    /** The Constant CLASS. */
    private static final String CLASS = "class"; //$NON-NLS-1$

    /** The Constant COMMITEE_REPORTS_CONTAIN. */
    private static final String COMMITEE_REPORTS_CONTAIN = "http://www.riksdagen.se/webbnav/?nid=3120&doktyp=betankande&bet"; //$NON-NLS-1$

    /** The Constant COMMITEE_REPORTS_PERIOD_2007_08. */
    private static final String COMMITEE_REPORTS_PERIOD_2007_08 = "http://www.riksdagen.se/webbnav/index.aspx?nid=3110&titel=&rm=2009%2F10&bet=&doktyp=bet%C3%A4nkande&org=&s=S%C3%B6k#t%22"; //$NON-NLS-1$

    /** The Constant DECISION. */
    private static final String DECISION = "Beslut:"; //$NON-NLS-1$

    /** The Constant DIV. */
    private static final String DIV = "div"; //$NON-NLS-1$

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory
            .getLog(CommitteeReportAgentImpl.class);

    /** The Constant NEXT. */
    private static final String NEXT = "nästa sida >"; //$NON-NLS-1$

    /** The Constant NEXT_VERSION2. */
    private static final String NEXT_VERSION2 = "nästa >"; //$NON-NLS-1$

    /** The Constant NORMAL. */
    private static final String NORMAL = "normal"; //$NON-NLS-1$

    /** The Constant PARLIAMENT_DECISION. */
    private static final String PARLIAMENT_DECISION = "Riksdagens beslut"; //$NON-NLS-1$

    /** The Constant SPAN. */
    private static final String SPAN = "span"; //$NON-NLS-1$

    /** The Constant TABLE. */
    private static final String TABLE = "table"; //$NON-NLS-1$

    /** The format. */
    private final SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd"); //$NON-NLS-1$

    /** The web client. */
    private final WebClient webClient;

    /**
     * Instantiates a new committee report agent impl.
     *
     * @param webClient the web client
     */
    public CommitteeReportAgentImpl(final WebClient webClient) {
        super();
        this.webClient = webClient;
        this.webClient.setJavaScriptEnabled(false);
    }


    /*
     * (non-Javadoc)
     * 
     * @see
     * com.hack23.cia.service.agent.sweden.CommiteeReportAgent#getCurrentList()
     */
    public final List<CommitteeReport> getCurrentList() throws Exception {
        List<CommitteeReport> resultat = new ArrayList<CommitteeReport>();

        DocumentAnswerPage answerPage = new DocumentAnswerPage(
                (HtmlPage) webClient.getPage(COMMITEE_REPORTS_PERIOD_2007_08));

        while (answerPage != null) {
            for (HtmlAnchor anchor : answerPage.getAnchors()) {

                if (anchor.getHrefAttribute()
                        .contains(COMMITEE_REPORTS_CONTAIN)) {

                    CommitteeReport commiteeReport = new CommitteeReport();
                    commiteeReport.setName(anchor.asText());
                    commiteeReport.setHref(anchor.getHrefAttribute());
                    resultat.add(commiteeReport);
                }
            }
            if (answerPage != null) {
                answerPage = answerPage.getNextPage();
            }
        }
        LOGGER.info("CommiteeReports found : " + resultat.size()); //$NON-NLS-1$
        return resultat;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * com.hack23.cia.service.agent.sweden.CommiteeReportAgent#getDecidedDateIfAny
     * (com.hack23.cia.model.sweden.CommiteeReport)
     */
    public final Date getDecidedDateIfAny(final CommitteeReport commiteeReport)
            throws Exception {
        HtmlPage page = (HtmlPage) webClient.getPage(commiteeReport.getHref());
        HtmlElement contentDiv = page.getDocumentElement()
                .getElementsByAttribute(DIV, CLASS, CENTER_PADDING).iterator()
                .next();

        List<HtmlElement> contentBlocks = contentDiv.getElementsByAttribute(
                SPAN, CLASS, NORMAL);

        LOGGER
                .info("Checking if decision has been made  " + commiteeReport.getHref()); //$NON-NLS-1$
        for (HtmlElement element : contentBlocks) {
            String str = element.asText().trim();
            if (str.startsWith(PARLIAMENT_DECISION)) {

                int startIndex = str.indexOf(DECISION);

                if (startIndex >= 0) {
                    String dateStr = str.substring(startIndex + 8,
                            startIndex + 18).replace("/", "-"); //$NON-NLS-1$ //$NON-NLS-2$

                    return parseDate(dateStr);
                } else {
                    return null;
                }
            }
        }
        return null;
    }

    /**
     * Parses the date.
     *
     * @param dateStr the date str
     * @return the date
     */
    private Date parseDate(final String dateStr) {
        try {
            return format.parse(dateStr);
        } catch (Exception pe) {
            LOGGER.warn("Problem parsing date ;" + dateStr, pe); //$NON-NLS-1$
        }
        return null;
    }
}
