/*
Copyright 2010 James Pether Sörling Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License. 
	$Id
*/

package com.hack23.cia.service.impl.agent.sweden;

import gnu.trove.THashMap;

import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.html.HtmlAnchor;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlTable;
import com.gargoylesoftware.htmlunit.html.HtmlTableCell;
import com.gargoylesoftware.htmlunit.html.HtmlTableRow;
import com.hack23.cia.model.sweden.impl.ParliamentMember;

/**
 * The Class ParliamentMemberAgentImpl.
 */
public class ParliamentMemberAgentImpl implements ParliamentMemberAgent {

    /** The Constant ANCHOR. */
    private static final String ANCHOR = "a"; //$NON-NLS-1$

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory
            .getLog(ParliamentMemberAgentImpl.class);

    /** The Constant PARLIAMENT_MEMBER_ENGLISH_WIKI_LIST. */
    private static final String PARLIAMENT_MEMBER_ENGLISH_WIKI_LIST = "http://en.wikipedia.org/wiki/List_of_members_of_the_Riksdag,_2006-2010"; //$NON-NLS-1$

    /** The Constant PARLIAMENT_MEMBER_LIST. */
    private static final String PARLIAMENT_MEMBER_LIST = "http://www.riksdagen.se/webbnav/index.aspx?fnamn=&enamn=&f_ar=&kn=&party=&electoralRegion=&rdlstatus=&org=&sort=&s=1&nid=1102"; //$NON-NLS-1$

    /** The Constant PARLIAMENT_MEMBER_SWEDISH_WIKI_LIST. */
    private static final String PARLIAMENT_MEMBER_SWEDISH_WIKI_LIST = "http://sv.wikipedia.org/wiki/Lista_%C3%B6ver_ledam%C3%B6ter_av_Sveriges_riksdag_2006-2010"; //$NON-NLS-1$

    /** The english wiki href map. */
    private Map<String, String> englishWikiHrefMap = new THashMap<String, String>();

    /** The href map. */
    private Map<String, String> hrefMap = new THashMap<String, String>();

    /** The web client. */
    private final WebClient webClient;

    /** The wiki href map. */
    private Map<String, String> wikiHrefMap = new THashMap<String, String>();

    /**
     * Instantiates a new parliament member agent impl.
     *
     * @param webClient the web client
     */
    public ParliamentMemberAgentImpl(final WebClient webClient) {
        super();
        this.webClient = webClient;
    }

    /**
     * Extract name.
     *
     * @param anchor the anchor
     * @return the string
     */
    private String extractName(final HtmlAnchor anchor) {
        String name = anchor.asText().replace(".", "");

        String[] split = name.trim().split(" "); //$NON-NLS-1$

        if (split.length == 2) {
            name = split[1] + ", " + split[0];
        } else {
            name = split[1] + " " + split[2] + ", " + split[0];
        }
        return name;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * com.hack23.cia.service.agent.sweden.ParliamentMemberAgent#getEnglishWikiHref
     * (com.hack23.cia.model.sweden.ParliamentMember)
     */
    @Override
    public final String getEnglishWikiHref(
            final ParliamentMember parliamentMember) {
        return englishWikiHrefMap.get(parliamentMember.getName());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * com.hack23.cia.service.agent.sweden.ParliamentMemberAgent#getHref(com
     * .hack23.cia.model.sweden.ParliamentMember)
     */
    @Override
    public final String getHref(final ParliamentMember parliamentMember) {
        return hrefMap.get(parliamentMember.getName());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * com.hack23.cia.service.agent.sweden.ParliamentMemberAgent#getWikiHref
     * (com.hack23.cia.model.sweden.ParliamentMember)
     */
    @Override
    public final String getWikiHref(final ParliamentMember parliamentMember) {
        return wikiHrefMap.get(parliamentMember.getName());
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.hack23.cia.service.agent.sweden.ParliamentMemberAgent#initData()
     */
    public final void initData() {
        try {
            HtmlPage htmlPage = (HtmlPage) webClient
                    .getPage(PARLIAMENT_MEMBER_LIST);
            List<HtmlAnchor> anchors = htmlPage.getDocumentElement()
                    .getHtmlElementsByTagName(ANCHOR);

            for (HtmlAnchor anchor : anchors) {
                hrefMap.put(anchor.asText(), anchor.getHrefAttribute());
                LOGGER.info("homepage:" + anchor.asText() + " - "
                        + anchor.getHrefAttribute());
            }
        } catch (Exception e) {
            LOGGER.warn("Problem Loading Parliament web site info", e); //$NON-NLS-1$
        }

        try {
            HtmlPage htmlPage = (HtmlPage) webClient
                    .getPage(PARLIAMENT_MEMBER_SWEDISH_WIKI_LIST);
            List<HtmlTable> tables = htmlPage.getDocumentElement().getElementsByAttribute("table", "id",
                            "sortable_table_id_0");
            
            if (tables != null) {
            	
            	if (tables.iterator().hasNext()) {
            	
		            HtmlTable table = tables.iterator().next();
		
		            List<HtmlTableRow> rows = table.getRows();
		
		            for (HtmlTableRow row : rows) {
		                if (row.getCells().size() > 1) {
		                    HtmlTableCell cell = row.getCell(1);
		
		                    List<HtmlAnchor> anchors = cell
		                            .getHtmlElementsByTagName(ANCHOR);
		
		                    HtmlAnchor anchor = null;
		                    if (anchors.size() > 0) {
		                        if (!cell.asText().contains("ersatt av")) {
		                            anchor = anchors.get(0);
		                        } else {
		                            anchor = anchors.get(anchors.size() - 1);
		                        }
		
		                        String name = extractName(anchor);
		
		                        String href = "http://sv.wikipedia.org"
		                                + anchor.getHrefAttribute();
		
		                        LOGGER.info("wiki sv: " + name + " - " + href);
		                        wikiHrefMap.put(name, href);
		
		                    }
		                }
		            }
            	}
            } else {
            	LOGGER.warn("Problem finding wiki links on page : " + PARLIAMENT_MEMBER_SWEDISH_WIKI_LIST );
            }

        } catch (Exception e) {
            LOGGER.warn("Problem Loading Parliament Swedish wiki site info", e); //$NON-NLS-1$
        }

        try {
            HtmlPage htmlPage = (HtmlPage) webClient
                    .getPage(PARLIAMENT_MEMBER_ENGLISH_WIKI_LIST);
            List<HtmlTable> tables = htmlPage.getDocumentElement()
                    .getElementsByAttribute("table", "class", "wikitable");

            HtmlTable table = tables.get(1);

            List<HtmlTableRow> rows = table.getRows();

            for (HtmlTableRow row : rows) {
                if (row.getCells().size() > 2) {
                    HtmlTableCell cell = row.getCell(2);

                    List<HtmlAnchor> anchors = cell
                            .getHtmlElementsByTagName(ANCHOR);

                    HtmlAnchor anchor = null;
                    if (anchors.size() > 0) {
                        if (!(cell.asText().contains("substituted") || cell
                                .asText().contains("replaced"))) {
                            anchor = anchors.get(0);
                        } else {
                            anchor = anchors.get(anchors.size() - 1);
                        }

                        String name = extractName(anchor);

                        String href = "http://en.wikipedia.org"
                                + anchor.getHrefAttribute();

                        LOGGER.info("wiki en: " + name + " - " + href);
                        englishWikiHrefMap.put(name, href);
                    }
                }
            }

        } catch (Exception e) {
            LOGGER.warn("Problem Loading Parliament English wiki site info", e); //$NON-NLS-1$
        }
    }
}
