/*
Copyright 2010 James Pether Sörling Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License. 
	$Id
*/

package com.hack23.cia.service.impl.common;

import java.util.List;

import com.hack23.cia.model.application.impl.common.ImageContent;
import com.hack23.cia.model.application.impl.common.Language;
import com.hack23.cia.model.application.impl.common.ParliamentChart;
import com.hack23.cia.model.sweden.impl.Ballot;
import com.hack23.cia.model.sweden.impl.CommitteeReport;
import com.hack23.cia.model.sweden.impl.ParliamentMember;
import com.hack23.cia.model.sweden.impl.ParliamentMemberVoteCompareResult;
import com.hack23.cia.model.sweden.impl.PoliticalParty;
import com.hack23.cia.model.sweden.impl.RegisterInformation;
import com.hack23.cia.model.sweden.impl.Vote;

/**
 * The Interface ParliamentService.
 */
public interface ParliamentService {

    /**
     * Find last votes.
     *
     * @param id the id
     * @return the list
     */
    List<Vote> findLastVotes(Long id);

    /**
     * Find last votes by ballot id.
     *
     * @param id the id
     * @return the list
     */
    List<Vote> findLastVotesByBallotId(final Long id);

    /**
     * Find parliament members by name.
     *
     * @param searchString the search string
     * @return the list
     */
    List<ParliamentMember> findParliamentMembersByName(String searchString);

    /**
     * Find political party by short code.
     *
     * @param party the party
     * @return the political party
     */
	PoliticalParty findPoliticalPartyByShortCode(String party);

    /**
     * Find votes.
     *
     * @param id the id
     * @return the list
     */
    List<Vote> findVotes(Long id);

    /**
     * Gets the all commitee reports.
     *
     * @return the all commitee reports
     */
    List<CommitteeReport> getAllCommiteeReports();

    /**
     * Gets the current list.
     *
     * @return the current list
     */
    List<ParliamentMember> getCurrentList();

    /**
     * Gets the current top list absent.
     *
     * @param number the number
     * @return the current top list absent
     */
    List<ParliamentMember> getCurrentTopListAbsent(int number);

    /**
     * Gets the current top list absent in party.
     *
     * @param i the i
     * @param party the party
     * @return the current top list absent in party
     */
    List<ParliamentMember> getCurrentTopListAbsentInParty(int i, String party);

    /**
     * Gets the current top list losers.
     *
     * @param i the i
     * @return the current top list losers
     */
    List<ParliamentMember> getCurrentTopListLosers(int i);

    /**
     * Gets the current top list losers in party.
     *
     * @param i the i
     * @param party the party
     * @return the current top list losers in party
     */
    List<ParliamentMember> getCurrentTopListLosersInParty(int i, String party);

    /**
     * Gets the current top list loyal.
     *
     * @param i the i
     * @return the current top list loyal
     */
    List<ParliamentMember> getCurrentTopListLoyal(int i);

    /**
     * Gets the current top list loyal in party.
     *
     * @param i the i
     * @param party the party
     * @return the current top list loyal in party
     */
    List<ParliamentMember> getCurrentTopListLoyalInParty(int i, String party);

    /**
     * Gets the current top list present.
     *
     * @param i the i
     * @return the current top list present
     */
    List<ParliamentMember> getCurrentTopListPresent(int i);

    /**
     * Gets the current top list present in party.
     *
     * @param i the i
     * @param party the party
     * @return the current top list present in party
     */
    List<ParliamentMember> getCurrentTopListPresentInParty(int i, String party);

    /**
     * Gets the current top list rebels.
     *
     * @param i the i
     * @return the current top list rebels
     */
    List<ParliamentMember> getCurrentTopListRebels(int i);

    /**
     * Gets the current top list rebels in party.
     *
     * @param i the i
     * @param party the party
     * @return the current top list rebels in party
     */
    List<ParliamentMember> getCurrentTopListRebelsInParty(int i, String party);

    /**
     * Gets the current top list winners.
     *
     * @param i the i
     * @return the current top list winners
     */
    List<ParliamentMember> getCurrentTopListWinners(int i);

    /**
     * Gets the current top list winners in party.
     *
     * @param i the i
     * @param party the party
     * @return the current top list winners in party
     */
    List<ParliamentMember> getCurrentTopListWinnersInParty(int i, String party);

    /**
     * Gets the graph.
     *
     * @param summary the summary
     * @param ballot the ballot
     * @param language the language
     * @return the graph
     */
	ImageContent getGraph(ParliamentChart summary, Ballot ballot,
			Language language);

    /**
     * Gets the graph.
     *
     * @param summary the summary
     * @param committeeReport the committee report
     * @param language the language
     * @return the graph
     */
	ImageContent getGraph(ParliamentChart summary, CommitteeReport committeeReport,
			Language language);

    /**
     * Gets the graph.
     *
     * @param chart the chart
     * @param language the language
     * @return the graph
     */
	ImageContent getGraph(ParliamentChart chart,Language language);

    /**
     * Gets the graph.
     *
     * @param chart the chart
     * @param parliamentMember the parliament member
     * @param language the language
     * @return the graph
     */
	ImageContent getGraph(ParliamentChart chart,ParliamentMember parliamentMember,Language language);

    /**
     * Gets the graph.
     *
     * @param summary the summary
     * @param politicalParty the political party
     * @param language the language
     * @return the graph
     */
	ImageContent getGraph(ParliamentChart summary, PoliticalParty politicalParty,
			Language language);

    /**
     * Gets the last decided commitee reports.
     *
     * @return the last decided commitee reports
     */
    List<CommitteeReport> getLastDecidedCommiteeReports();

    /**
     * Gets the parliament member.
     *
     * @param parliamentMemberId the parliament member id
     * @return the parliament member
     */
    ParliamentMember getParliamentMember(Long parliamentMemberId);

	/**
	 * Gets the parliament member enemies.
	 *
	 * @param parliamentMember the parliament member
	 * @return the parliament member enemies
	 */
    List<ParliamentMemberVoteCompareResult> getParliamentMemberEnemies(
            ParliamentMember parliamentMember);

	/**
	 * Gets the parliament member friends.
	 *
	 * @param parliamentMember the parliament member
	 * @return the parliament member friends
	 */
    List<ParliamentMemberVoteCompareResult> getParliamentMemberFriends(
            ParliamentMember parliamentMember);
	
	
	/**
	 * Gets the parliament member party enemies.
	 *
	 * @param parliamentMember the parliament member
	 * @return the parliament member party enemies
	 */
    List<ParliamentMemberVoteCompareResult> getParliamentMemberPartyEnemies(
            ParliamentMember parliamentMember);

	/**
	 * Gets the parliament member party friends.
	 *
	 * @param parliamentMember the parliament member
	 * @return the parliament member party friends
	 */
    List<ParliamentMemberVoteCompareResult> getParliamentMemberPartyFriends(
            ParliamentMember parliamentMember);
	
	/**
	 * Gets the political parties.
	 *
	 * @return the political parties
	 */
	List<PoliticalParty> getPoliticalParties();

	/**
	 * Load ballot by id.
	 *
	 * @param ballotId the ballot id
	 * @return the ballot
	 */
	Ballot loadBallotById(Long ballotId);

	/**
	 * Load committee report by id.
	 *
	 * @param committeeReportId the committee report id
	 * @return the committee report
	 */
    CommitteeReport loadCommitteeReportById(Long committeeReportId);

	/**
	 * Lookup register information.
	 *
	 * @param parliamentMember the parliament member
	 * @return the register information
	 */
    RegisterInformation lookupRegisterInformation(
            ParliamentMember parliamentMember);
}
