/*
Copyright 2010 James Pether Sörling Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License. 
	$Id
*/

package com.hack23.cia.web.controller.application;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import thinwire.ui.Button;
import thinwire.ui.Component;
import thinwire.ui.Frame;
import thinwire.ui.Hyperlink;
import thinwire.ui.Menu;
import thinwire.ui.Tree;
import thinwire.ui.GridBox.Range;
import thinwire.ui.event.ActionEvent;
import thinwire.ui.event.KeyPressEvent;

import com.hack23.cia.model.application.impl.user.ApplicationActionEvent;
import com.hack23.cia.web.action.common.ApplicationErrorAction;
import com.hack23.cia.web.action.common.ControllerAction;
import com.hack23.cia.web.action.common.ControllerActionListener;

/**
 * The listener interface for receiving applicationAction events.
 * The class that is interested in processing a applicationAction
 * event implements this interface, and the object created
 * with that class is registered with a component using the
 * component's <code>addApplicationActionListener<code> method. When
 * the applicationAction event occurs, that object's appropriate
 * method is invoked.
 *
 * @see ApplicationActionEvent
 */
public class ApplicationActionListener implements ControllerActionListener {

    /** The Constant LOGGER. */
    private static final Log LOGGER = LogFactory
            .getLog(ApplicationActionListener.class);

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1L;

    /** The controller. */
    private final Controller controller;

    /**
     * Instantiates a new application action listener.
     *
     * @param controller the controller
     */
    public ApplicationActionListener(final Controller controller) {
        super();
        this.controller = controller;
    }

    /*
     * (non-Javadoc)
     * 
     * @seethinwire.ui.event.ActionListener#actionPerformed(thinwire.ui.event.
     * ActionEvent)
     */
    public final void actionPerformed(final ActionEvent ev) {
        Object source = ev.getSource();
        Object userObject = null;

        if (source instanceof Button) {
            userObject = ((Button) source).getUserObject();
        } else if (source instanceof Hyperlink) {
            userObject = ((Hyperlink) source).getUserObject();
        } else if (source instanceof Range) {
            userObject = ((Range) source).getRow().getUserObject();
        } else if (source instanceof Menu.Item) {
            if (((Menu.Item) source).isEnabled()) {
                userObject = ((Menu.Item) source).getUserObject();
            }
        } else if (source instanceof Frame) {
            userObject = ((Frame) source).getUserObject();
        } else if (source instanceof Tree) {
            if (((Tree) source).getSelectedItem() != null) {
                userObject = ((Tree) source).getSelectedItem().getUserObject();
            } else {
                userObject = ((Tree) source).getUserObject();
            }
        } else if (source instanceof Tree.Item) {
            userObject = ((Tree.Item) source).getUserObject();
        }

        if (userObject != null) {
        	handleUserAction(userObject);
        }
    }

    /* (non-Javadoc)
     * @see com.hack23.cia.web.action.common.ControllerActionListener#handleUserAction(java.lang.Object)
     */
    public final void handleUserAction(final Object userObject) {
        if (userObject != null) {
            try {
                controller.handleAction((ControllerAction) userObject);
            } catch (Exception e) {
                LOGGER.error("Should have done rollback, unexpected error:", e); //$NON-NLS-1$

                ApplicationErrorAction applicationErrorAction = new ApplicationErrorAction(e.getStackTrace(),"Should have done rollback, unexpected error");
                try {
                    controller.handleAction(applicationErrorAction);
                } catch (Exception reallyScrewedUpException) {
                    LOGGER
                            .fatal(
                                    "Something really bad, failed to log application error",
                                    reallyScrewedUpException);
                }
            }
        }
    }

	/*
     * (non-Javadoc)
     * 
     * @see
     * thinwire.ui.event.KeyPressListener#keyPress(thinwire.ui.event.KeyPressEvent
     * )
     */
    @Override
    public final void keyPress(final KeyPressEvent ev) {
        Object source = ev.getSource();
        Object userObject = null;

        if (source instanceof Component) {
            userObject = ((Component) source).getUserObject();
        }

        if (userObject != null) {
        	handleUserAction(userObject);
        }
    }
}
